function [v_sol status extra] = FBA(S , Lbound, Ubound , OBJ, maximize)
%[v_sol status] = FBA(S , Lbound, Ubound , OBJ, [maximize] )
%               or
%[v_sol status] = FBA( model, [maximize] )
% Takes as input : (or model must have )
%	S 		 - The stoichiometric matrix, rows are metabolites and columns are reactions
%	Ubound	 - A vector the same length as the number of reactions, indicating the upper bound for each
%	Lbound	 - A vector the same length as the number of reactions, indicating the lower bound for each
%	OBJ		 - A vector the same length as the number of reactions, indicating which fluxes should be optimized
%
% Optional input:	
%    maximize - A flag as to maximize( defalut or 1) or minimize ( 0 ) 
%
%	
% Returns : The fluxes through each reaction and the exit status of glpk.
%
% Author: Evan
% Date:   ??
% Updated:  Niels 2/18/2008 - removed unused vars
%           Niels May 6  2008 - allowed for selecting maximization/minimization
%           Niels Nov 23 2008 - allowing for models to be added as a single
%           variable.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Check if a model was passed
if ( isstruct( S ) )
    if ( isfield( S, 'S' ) && isfield( S, 'lb') ...
     &&  isfield( S, 'ub') && isfield( S, 'c' ) )

         model = S;

         % check if a maximize was included 
         if (exist( 'Lbound', 'var' ) && isnumeric( Lbound ) )
             maximize = Lbound;
         end

         % get needed fields
         S      = model.S;
         Lbound = model.lb;
         Ubound = model.ub;
         OBJ    = model.c;

   else
         error( 'Unrecognized model: must contain S, lb, ub and c fields' );
   end
end

%%%ADJUST STOICHIMOETRIC MATRIX AND B-VECTOR TO IMPOSE EXACT CONSTRAINTS%%%

%NUMBER OF METABOLITES (ROWS OF S, CONSTRAINTS) and
%NUMBER OF REACTIONS (COLUMNS OF S, VARIABLES)
[M,N] = size(S);

%SET THE B VECTOR FOR THE RIGHT HAND SIDE OF EQUATION, Sv= 0
Bvector = zeros(M, 1);
	
	
%%%SET PARAMETERS FOR THE LP OPTIMIZATRION SOLVER%%%
	
%GENERATES A STRING OF SYMBOLS 'S', INDICATING THE TYPE OF CONSTRAINTS (<,>,=)
%(IN STIOCH*V = 0) USED IN THE OPTIMIZER. THE FOLLOWIN SETTING IS DEFAULT IN FBA
CTYPE = char('S'*ones(1,M))';

%GENERATES A STRING OF SYMBOLS 'C', INDICATING THE TYPE OF VARIABLES
%'C' MEANS CONTINUOUS VARIABLES (STANDARD IN FBA)
VARTYPE = char('C'*ones(1,N))';

%IF SET TO 3, PRINTS OUT DETAILS OF OPTIMIZATION PROCESS
param.msglev = 0;
	
%TURN OFF THE PRESOLVER
param.presol = 1;
    
%TURN OFF Scalling (0) geo then eq 3
param.scale = 3;

% IMPORTANT: THIS, WHEN SET TO 2, USES INTERIOR POINT, PREVENTING A GLPKMEX BUG 
%	     FROM HAPPENING (I.E. A SITUATION WHEN 1<=V<=1 IS INTERPRETED AS 1<V<1, AND 
%	     CONSIDERED INFEASIBLE).
param.lpsolver = 1; 
    
%max time to solve...
%param.tmlim = 10000;

%max number of iterations to solve
param.itlim = 10000;
    
%IF 1, SAVE PROBLEM IN LP FORMAT
param.save_pb = 0; 

% -1 MAXIMIZES, 1 MINIMIZES
if( ~ exist( 'maximize', 'var' ) || maximize == 1 )
    min_max = -1;
elseif( maximize == 0 )
    min_max = 1;
else
    error( 'Maximize variable must be left blank, 1, or 0.' );
end

%%%RUN OPTIMIZATION AND DISPLAY SOME RESULTS%%%
[v_sol,obj_sol,status,extra] = glpk(OBJ , S , Bvector , Lbound , Ubound , CTYPE, VARTYPE , min_max , param);

%retry with presolver if FBA fails...    
if (status ~= 5 )
    param.presol = 0;
    [v_sol,obj_sol,status,extra] = glpk(OBJ , S , Bvector , Lbound , Ubound , CTYPE, VARTYPE , min_max , param);
end

