function [ int_type, bm_values ] = check_interaction( model, media )
%   usage: [ int_type, bm_values ] = check_interaction( model, media )
%   purpose: determine the interaction type for a model and media set
%   inputs:
%       model   - a constraint based model with the following fields
%           .t_rxns - vector indicating shuttle rxns and who they belong to
%           .lb - lower bound of fluxes
%           .ub - upper bound of fluxes
%           .bm - biomass vector (optoinal,default is 1) => bm is fixed to 1/10 this value
%           .S  - stoichiometric matrix
%           .c  - objective vector
%       media   - vector of extracelluar metabliies to use as media    
%
%   results:
%       0 => media is not viable for both orgaisms!
%       1 => A => B
%       2 => B => A
%       3 => A; B
%       4 => A <==> B
%   Author: Niels
%   Date: June 9, 2010
%   Note:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


[undef, env_met] = get_environment(  model );
bm_values = zeros(1,4);

model.lb(env_met) = 0;
model.lb( media ) = -1000;

BM = find( model.c );
M1R = intersect( find( model.lb > 0 ), find( model.m_rxns == 1 ) );
M2R = intersect( find( model.lb > 0 ), find( model.m_rxns == 2 ) );

if (isfield( model, 'bm') )
    model.lb(BM) = model.bm/10;
else
    model.lb(BM) = .1;
end
    
% confirm each model can grow in the joined model
model.c(BM(2)) = 0;
[vj1,ej1] = FBA( model );

if ( ej1 == 5 )
    bm_values(1) = vj1(BM(1));
else
    bm_values(1) = -ej1;
end

model.c(BM(2)) = 1;
model.c(BM(1)) = 0;
[vj2,ej2] = FBA( model ); 

if ( ej2 == 5 )
    bm_values(2) = vj2(BM(2));
else
    bm_values(2) = -ej2;
end

model.c(BM(1)) = 1;

%check org1 alone

model_1 = model;
model_1.lb( logical(model.t_rxns == 2 ) ) = 0;
model_1.ub( logical(model.t_rxns == 2 ) ) = 0;
model_1.lb( BM(2) ) = 0;
model_1.c(  BM(2) ) = 0;
model_1.lb( M2R ) = 0;

[v1,e1] = FBA( model_1 );

if ( e1 == 5 ) 
    bm_values(3) = v1(BM(1));
else
    bm_values(3) = -e1;
end

%check org2 alone
model_2 = model;
model_2.lb( logical(model.t_rxns == 1 ) ) = 0;
model_2.ub( logical(model.t_rxns == 1 ) ) = 0;
model_2.lb( BM(1) ) = 0;
model_2.c( BM(1) ) = 0;
model_2.lb( M1R ) = 0;

[v2,e2] = FBA( model_2 );

if ( e2 == 5 )
    bm_values(4) = v2(BM(2));
else
    bm_values(4) = -e2;
end


% media failed for pair!
if ( ej1  ~=  5 || ej2  ~=  5 )
    int_type = 0;
elseif ( e1 == 5 && e2 ~= 5 )
    int_type = 1;
elseif ( e1 ~= 5 && e2 == 5 )
    int_type = 2;
elseif ( e1 == 5 && e2 == 5 )
    int_type = 3;
elseif ( e1 ~= 5 && e2 ~= 5 )
    int_type = 4;
else

end

