function [MM,  status, v_sol, new_S, model ] = find_min_media( model )
%[MM, status] = find_min_media( model )
%   Purpose: finds a minimal media condition for a given model
%       Extra model params:
%           media:  binary media vector (size of lb) containing media to select from
%               levels of metabolites not in media can be set via lb.
%           bm:     fesible biomass has been determined preiously and is set with .bm
%   Author: Niels
%   Date:
%   Notes: Cleaned up some code 10/7/13 -nk
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Max_Media_flux = 100;

% asigning params from model
S  = model.S;
lb = model.lb;
ub = model.ub;
c  = model.c;

% bm is the maximal biomass of the model
if ( isfield( model, 'bm' ) )
    bm = model.bm;
else
    bm = 1;
end

% If media to select from is not defined, fix it to all extracellular media
if ( isfield( model, 'media' ) )
    media = model.media;
else
    [xx, m_env] = get_environment( model, 0 );
    media = zeros( size(model.lb) );
    media(m_env) = 1;
end

% open up media constrains (assmed to be controled with LB)
lb( logical(media) ) = -Max_Media_flux;

% fix model to produce at least some fraction of biomass
lb( logical(c) ) = bm/10;

%NUMBER OF METABOLITES M (ROWS OF S, CONSTRAINTS)
%NUMBER OF REACTIONS N (COLUMNS OF S, VARIABLES)
[M,N] = size(S); 

% number of metabolites to choose from	
MS = length( find( media ) );

%SET THE B VECTOR FOR THE RIGHT HAND SIDE OF EQUATION, Sv= 0
Bvector = zeros( M, 1 );
	
%set the cost matrix for each metabolite in media
Media_Mat = - eye(N);
Media_Mat = Media_Mat( logical(media), :);
rev_Media_Mat = zeros(MS);
for i=1:MS
    rev_Media_Mat(i,i) = -Max_Media_flux;
end
	
%need to combine S mat!! and update lb and ub
new_S = [ [ S,  zeros( M, MS) ]; ...
          [Media_Mat, rev_Media_Mat ] ];

%update lb, ub, and B_vector
lb = [lb; zeros(MS,1)];
ub = [ub; ones(MS, 1)];

Bvector = [Bvector; zeros(MS,1) ];

%Need to define new C
new_c = zeros( size( new_S, 2), 1);
new_c( ( 1:MS ) + N ) = 1;      

% set variable tyes and bounding type

%GENERATES A STRING OF SYMBOLS INDICATING THE TYPE OF CONSTRAINTS (<,>,=)
% S - =
% U - <
% L - >
%(IN STIOCH*V = 0) USED IN THE OPTIMIZER. THE FOLLOWIN SETTING IS DEFAULT IN FBA
CTYPE = [ char('S'*ones(1,M)), char('U'*ones(1,MS)) ]';

%GENERATES A STRING OF SYMBOLS 'C', INDICATING THE TYPE OF VARIABLES
%'C' MEANS CONTINUOUS VARIABLES (STANDARD IN FBA)
VARTYPE = [char('C'*ones(1,N)), char( 'I'*ones(1,MS) ) ]';


%%%SET PARAMETERS FOR THE LP OPTIMIZATRION SOLVER%%%

%price (default: 1). Pricing option (for both primal and dual simplex):
%                    0 - Textbook pricing.
%                    1 - Steepest edge pricing
%param.price = 0;

%param.relax = 10e-10;

%IF SET TO 3, PRINTS OUT DETAILS OF OPTIMIZATION PROCESS
param.msglev  = 0;

%TURN OFF THE PRESOLVER (0) or ON (1)
param.presol = 1;

% more params to tweek
param.tolbnd = 1e-9; % Relative tolerance used to check ifthe current basic solution is primal feasible
param.toldj  = 1e-9; % Absolute tolerance used to check if the current basic solution is dual feasible
param.tolint = 1e-9; % Relative tolerance used to check if the current basic solution is integer feasible

%Selections what type of cutting planes we apply to the space (default = 1) 
% 0 => off
% 5 => all cuts
param.usecuts = 5;

%Relative tolerance used to check if the current basic solution is integer
%feasible ( Default = 10e-5 )
param.tolint = 10e-12;

%scale problem befor solving?
param.scale = 0;

% time limit for mips
%param.tmlim = -1;

%IF 1, SAVE PROBLEM IN LP FORMAT
param.save_pb = 0; 

% -1 MAXIMIZES, 1 MINIMIZES -> default is to minimize
if( ~ exist( 'maximize', 'var' ) || maximize == 0 )
    min_max = 1;
elseif( maximize == 1 )
    min_max = -1;
else
    ERROR( 101, 'Maximize variable must be left blank, 1, or 0.' );
end

model.lb = lb;
model.c  = new_c;
model.S  = new_S;
model.ub = ub;

%%%RUN OPTIMIZATION AND DISPLAY SOME RESULTS%%%

[v_sol,obj_sol,status ] = glpk(new_c , new_S , Bvector , lb , ub , ... 
                               CTYPE, VARTYPE , min_max , param);

% try differenct cuts to find alt solution if fail
if ( status == 105 )
    
    param.usecuts = 3;
    [v_sol,obj_sol,status ] = glpk(new_c , new_S , Bvector , lb , ub , ... 
                               CTYPE, VARTYPE , min_max , param);
                           
    if ( status == 105 )
        sprintf 'Use of alt cuts did not work\n';
    end
end
      
%find only those parts we want 
mm_num = find( media );
MM = mm_num( logical(v_sol( logical(new_c) )) );


end
