function [ environment, env_rxn_no ] = get_environment( model, current, lb, ub )
%[ environment, env_rxn_no ] = get_environment( model [, current, lb, ub] )
%   gets all rxns with that make up the 'enviroment' (media) of a model
%   does this by finding all the rxns with 'EX_' that are available to the
%   organism
%
%   inputs:
%       model   is a palsson style model
%       current is (0 or 1, defalut = 1) optional param to indicate if you want 
%               the current evironment (1: ex_rxns < 0), or all (0: all ex_rxns)
%       lb      include to over-ride model lower bound
%       ub      include to over-ride model upper bound
%   outputs:    
%       environment vector of strings of enviroment rxn and values:
%           rxn_num,media_num,rxn_name,met_name,lower bound,upper bound
%       env_rxn_no  vector of doubles indicateing rxn number
%
%   Auther:  Niels
%   Date:    15 July 2008
%   Updated  Sept 15, 2009 -> changed eviroment output format
%   Notes:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%variable setup
S    = model.S;
if ( ~ exist( 'lb', 'var' ) || length( lb ) ~= size(S,2) )
    lb   = model.lb;
end

if ( ~ exist( 'ub', 'var' ) || length( ub ) ~= size(S,2) )
    ub   = model.ub;
end

rxns = model.rxns;
mets = model.mets;

% by default return the current environment
if ( ~exist( 'current', 'var' ) )
    current = 1;
end
    
 
environment  = {};
current_only = zeros( 0,1 );
env_rxn_no = zeros( 0, 1);

expression = '^EX_'; 

for i=1:length(lb)
    % check if rxn is external (possible medium compostion)
    if ( regexpi( rxns{i}, expression ) )
        met = find( S(:,i) );
        coefficent = S(met,i);
        
        % is it in the media?

        position = length(environment) + 1;
        environment{position,1} = sprintf( '%d,%d,%s,%s,%d,%d', ...
            i, position, char(rxns{i}), char(mets{met}), lb(i), ub(i) );
        env_rxn_no(position) = i;
        current_only( position,1 ) = 0;
        
        % determine if there is currently any in the medium
        if ( max( coefficent*(lb(i)), coefficent*(ub(i)) ) > 0 )
            current_only( position,1 ) = 1;
        end

        
    end
end

% if current only wanted, just return that
if ( current )
    environment = environment( logical(current_only) );
    env_rxn_no  = env_rxn_no( logical(current_only) );
end

end
