function [ joined_model ] = join_models( models, org_names)
%[ joined_model ] = join_models( models, org_names)
%   Joins two or more stoichiometric models into a joint model.
%   each model maitains it's own extracellular space which is joined to
%   the rest of the model through a joint extracelluar space
%   input :
%       models: Cell array of COBRA style stoichiometric models
%       org_names: Cell array of names for each model in models
%   output:
%       joined_model: Single model containing all sub-models in models
%           shuttle reactions are idicated 
%           org_specific ractions are indicated
%
%   Auther: Niels
%   Date:   Sept 28 2009
%   Notes:  Renamed to join_models from join_models_v3 10/7/13
%           added metNames support
%           Cleaned up code
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% upper and lower bounds for the joint model, note this can not be inf/-inf
% because we are multiplying it by a matrix...
max_ub =  1000;
min_lb = -1000;

% create model names if they don't exists yet...
if ( ~ exist( 'org_names', 'var' ) || length(models) ~= length(org_names))
    org_names = cell( length(models),1 );
    for i=1:length(models)
        org_names{i} = sprintf( 'org_%d', i );
    end
end

%% first join external metabolites
% defalult compartents to collaps on
compartments = 'e';
media_mets   = {};  %for metabolite symbols
media_mf     = {};  %for metabolite formulas
media_mn     = {};  %for metabolite names

total_n = 0;

for i=1:length( models )
    model = models{i};

    [m,n] = size( model.S );
   
    total_n = total_n + n;

    % need to get extracellular/exchange mets only....
    [xxx, ex_rxns] = get_environment( model, 0 ); 
    
    % determine non-exchange reactions for org
    org_rxns = setdiff( 1:n, ex_rxns);
    
    mets      = cell(length(ex_rxns),1); %for metabolite symbols
    met_forms = cell(length(ex_rxns),1); %for metabolite formulas
    met_names = cell(length(ex_rxns),1); %for metabolite names
    % get mets, now in same order as exchange reactions
    for j=1:length(ex_rxns)
        mets{j} = model.mets{ logical( model.S(:,ex_rxns(j)) ) };
  
        % assign formula
        if( isfield( model, 'metFormulas' ) )
            met_forms{j} = model.metFormulas{ logical( model.S(:,ex_rxns(j)) ) };
        else
            met_forms{j} = '';
        end
    
        % assign name
        if( isfield( model, 'metNames' ) )
            met_names{j} = model.metNames{ logical( model.S(:,ex_rxns(j)) ) };
        else
            met_names{j} = '';
        end
        
        %new name for met
        models{i}.rxns{ex_rxns(j)} = ['T_' mets{j}];
    end

    [met_map] = map_mets( mets, media_mets, compartments, compartments );

    % add any mets not yet in the media
    for j=1:size(met_map,1)
        if ( ~ any( met_map(j,:) ) )

            next_media_met = size(media_mets,1) + 1;
            media_mets{ next_media_met ,1} = mets{j};
            met_map(j,size(met_map,2) + 1) = 1;
            
            if( isfield( model, 'metFormulas' ) )
                media_mf{next_media_met ,1} = met_forms{j};
            else
                media_mf{next_media_met ,1} = '';
            end
    
            if( isfield( model, 'metNames' ) )
                media_mn{next_media_met ,1} = met_names{j};
            else
                media_mn{next_media_met ,1} = '';
            end
            
        elseif ( isempty( media_mf{find(met_map(j,:))} ) )
        
            if( isfield( model, 'metFormulas' ) )
                media_mf{next_media_met ,1} = met_forms{j};
            end

            if( isfield( model, 'metNames' ) )
                media_mn{next_media_met ,1} = met_names{j};
            end
            
        end
        
    end

    models{i}.metMap = met_map;
    models{i}.t_rxns = ex_rxns;
    models{i}.m_rxns = org_rxns;
    
end

total_n = total_n + length(media_mets);

%base model params....
new_S  = - eye( length(media_mets) );
new_lb = zeros( length(media_mets), 1 ); 
new_ub = max_ub * ones(  length(media_mets), 1 ); 
new_c  = zeros( length(media_mets), 1 );
new_mets = media_mets;
new_mf   = media_mf;
new_mn   = media_mn;
trans_rxns = zeros( total_n, 1 );
org_rxns   = zeros( total_n, 1 );
met_maps   = zeros(  length(media_mets), length( models ) );

new_rxns = cell(length(media_mets),1);
for i=1:length(media_mets)
    rxn_name = ['EX_' new_mets{i}];
    new_rxns{i} = rxn_name;
end



%% build new S matrix and update the rest of the model params
for i=1:length( models )
    model = models{i};
    S = model.S;

    [m1,n1] = size( new_S );
    [m2,n2] = size( S );

    new_S = [ new_S, zeros( m1,n2 ); zeros( m2, n1 ), S ];
    
    % update ex of model to be transfer reactions
    ex_rxns = model.t_rxns;
    met_map = model.metMap;
    met_pos = zeros(length(ex_rxns),1);
    for j=1:length( ex_rxns )
        met_pos(j) = find( met_map(j,:) );
        rxn = ex_rxns(j);
        new_S( met_pos(j), rxn+n1  ) = 1;
        met_maps(met_pos(j),i) = rxn;
    end

    % update old exchange rxns and new exchange rxns..
    
    new_lb(met_pos) = new_lb(met_pos) + model.lb( model.t_rxns );
    model.lb( model.t_rxns ) = min_lb;
    
    % make new constriants...
    new_lb = [ new_lb; model.lb ];
    new_ub = [ new_ub; model.ub ];
    new_c  = [ new_c; model.c  ];    
    
    % make new labels
    new_rxns = [new_rxns; model.rxns];
    new_mets = [new_mets; model.mets];
    
    if( isfield( model, 'metFormulas' ) )
        new_mf = [new_mf; model.metFormulas];
    else
        mf_size = length( new_mf );
        
        for k=1:m2
            new_mf{mf_size + k} = '';
        end
    end
   
    if( isfield( model, 'metNames' ) )
        new_mn = [new_mn; model.metNames];
    else
        mn_size = length( new_mn );

        for k=1:m2
            new_mn{mn_size + k} = '';
        end
    end
 
    % keep track of what belongs to whom
    trans_rxns( n1 + model.t_rxns ) = i;
    org_rxns( n1 + model.m_rxns ) = i;

end


%% Build empty structure to put values back into model format
joined_model = struct;

joined_model.S      = new_S;
joined_model.mets   = new_mets;
joined_model.metNames = new_mn;
joined_model.metFormulas = new_mf;
joined_model.rxns   = new_rxns;
joined_model.lb     = new_lb;
joined_model.ub     = new_ub; 
joined_model.c      = new_c;
joined_model.model_names = org_names; %ordered list of org names
joined_model.t_rxns = trans_rxns; %transfer rxns marking per org
joined_model.m_rxns = org_rxns; %model specific rxns per org
joined_model.ex_met_maps = met_maps; % maps new ex rxns to old per org

end



