function [ex_mets status v_ex ] = min_exchange_limited( model, met_space )
%[ex_mets status v_ex] =  min_exchange_limited( model )
% Purpose:  Identifies a set metabolties that are can be excahgne which 
%           contain the fewest number of metabolites
%           This is done by finding those metabolites for with the flux into a 
%           sub model is positive (from env), and negative for the other.  This 
%           can only occure if exchange of the metaoblite is occuring.
%           This script is used to search only in the reduced space of excahgne
%           as found in met_sapce
%
% Takes as input a modified COBRA stoichiometric metabolic model: (or model must have )
%	S 		- The stoichiometric matrix, rows are metabolites and columns are reactions
%	Ubound	- A vector the same length as the number of reactions, indicating the upper bound for each
%	Lbound	- A vector the same length as the number of reactions, indicating the lower bound for each
%	OBJ		- A vector the same length as the number of reactions,
%             indicating which fluxes should be optimized
%   t_rxns  - A vector indicate wich reactions are shuttle reactions for each sub model
%   ex_met_map - A map of exchange metabolites.
%   met_space - Allow interations only in this space of metaoblites 
%
% Returns : 
%   ex_mets: - a minimal set of exchanged metabolites.
%   status:  - return status of glpk
%   v_ex:    - fesible flux solution using these exchanged metabolites
%
% Author: Niels
% Date:     Aug 20 19/2010
% Updated:  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	

MAX = 1000;

    %Check if a model was passed
    if ( isstruct( model ) )
       if ( isfield( model, 'S') && isfield( model, 'lb')  && ... 
            isfield( model, 'ub') && isfield( model, 'c' ) && ...
            isfield( model, 't_rxns') && isfield( model, 'ex_met_maps' ) )
           
           % get needed fields
           S      = model.S;
           Lbound = model.lb;
           Ubound = model.ub;
           OBJ    = model.c;
           t_rxns = model.t_rxns;
           ex_met_map = model.ex_met_maps;
           
       else
           error( 'Unrecognized model: must contain S, lb, ub and c fields' );
       end
    else
        error( 'min_exchange expacts a joined model to be passed' );
    end

    %fix minimal biomass production
    if ( isfield( model, 'bm' ) )
        Lbound(logical( OBJ )) = model.bm/10;
    else
        Lbound(logical( OBJ )) = .1;
    end
    
    Lbound(logical( OBJ ))
    
    % media
    [~,media]   = get_environment(model);
    %[~,env_mets = get_environment(model, 0);
    
    
    %find all exchange-able mets 
    ex_mets_all = find( all(ex_met_map, 2) );
    if ( ~ exist( 'met_space', 'var') || isempty(met_space) )
        met_space = ex_met_all;
    end
    ex_mets = setdiff(intersect( ex_mets_all, met_space ), media); % metabolites to minize over
    off_mets = setdiff(setdiff( ex_mets_all, met_space ), media);  % metaboltes not to allow exchange of
    
    % all transfer reactions for org1/2
    all_o1_t_rx = find( t_rxns == 1 );
    all_o2_t_rx = find( t_rxns == 2 );

    %vector for pairied transfer reations...
    o1_t_rx = zeros( size( ex_mets ) );
    o2_t_rx = zeros( size( ex_mets ) );

    %map paired metabolite shuttle reactions
    for i=1:length( ex_mets )
        ex_met = ex_mets(i); 
        met_rxns = find( S(ex_met,: ) );
        o1_t_rx(i) = intersect( met_rxns, all_o1_t_rx);
        o2_t_rx(i) = intersect( met_rxns, all_o2_t_rx);
    end
    
    %find t rxns for off_mets! (by not allowing import of met into the
    %sub-model, we are effectively turning off the exchange of a metoblite
    for i=1:length( off_mets )
        off_met = off_mets(i); 
        met_rxns = find( S(off_met,: ) );
        Lbound( intersect( met_rxns, all_o1_t_rx) ) = 0;
        Lbound( intersect( met_rxns, all_o2_t_rx) ) = 0;
    end

    %construct new_S matrix
    
    [M,N] = size(S); %size of orginal S amtrix
    t = length( o1_t_rx ); %number of transer reactions
    cost_S = eye(N); 

    new_S = [S,                 zeros(M,5*t )                              ;... % basic model
             cost_S(o1_t_rx,:), -MAX*eye(t),  zeros(t,4*t)                 ;... % id positive flux through o1_t rxns
            -cost_S(o1_t_rx,:), zeros(t),     -MAX*eye(t),  zeros(t,3*t)   ;... % id negative flux through o1_t rxns
             cost_S(o2_t_rx,:), zeros(t,2*t), -MAX*eye(t),  zeros(t,2*t)   ;... % id positive flux through o2_t rxns
            -cost_S(o2_t_rx,:), zeros(t,3*t), -MAX*eye(t),  zeros(t)       ;... % id negative flux through o2_t rxns
             zeros(t,N),        eye(t),       zeros(t,t*2), eye(t),  -eye(t); ... % determine if ( +, -) ie exchange A to B
             zeros(t,N),        zeros(t),     eye(t),       eye(t),  zeros(t), -eye(t) ]; %determine if ( -, + ) ie exchange B to A

     
    new_LB = [ Lbound; zeros( t*5, 1) ];
    new_UB = [ Ubound; ones(  t*5, 1) ];

    %SET THE B VECTOR FOR THE RIGHT HAND SIDE OF EQUATION, Sv= 0, 
    Bvector = [zeros( M + 4*t , 1); ones(2*t,1)] ;
	
    %SET THE OBJ vector
    OBJ = [zeros( N + 4*t , 1); ones(t,1)] ;
	
	%% SET PARAMETERS FOR THE LP OPTIMIZATRION SOLVE %%
	
	%GENERATES A STRING OF SYMBOLS 'S', INDICATING THE TYPE OF CONSTRAINTS (<,>,=)
	%(IN STIOCH*V = 0) USED IN THE OPTIMIZER. THE FOLLOWIN SETTING IS DEFAULT IN FBA
    CTYPE = [char('S'*ones(1,M)) char('U'*ones(1,6*t) )]';

	%GENERATES A STRING OF SYMBOLS 'C', INDICATING THE TYPE OF VARIABLES
	%'C' MEANS CONTINUOUS VARIABLES (STANDARD IN FBA)
	VARTYPE = [char('C'*ones(1,N)) char('B'*ones(1,5*t) )]'; 

	%IF SET TO 3, PRINTS OUT DETAILS OF OPTIMIZATION PROCESS
	param.msglev = 3;
	
	% IMPORTANT: THIS, WHEN SET TO 2, USES INTERIOR POINT, uses simplex 
	param.LPsolver = 1; 

	%IF 1, SAVE PROBLEM IN LP FORMAT
	param.save_pb = 0; 

    % set LP to maximize/minimize if maximize parameter passed, else just maximize
	% -1 MAXIMIZES, 1 MINIMIZES
    min_max = 1;

	%TURN OFF THE PRESOLVER
	param.presol = 1;
    %param.binarize = 1;
    
    %Use cuts (0=off, 5=all on)
    param.usecuts = 5;

    % number of iterations to look at before giving up!
    %param.itlim = 1000;
    %param.tmlim = 1000;

    %param.branch = 2;

    %param.btrack = 3;
    
    
    param.tolint = 1e-10;

	%%%RUN OPTIMIZATION%%%
	[v_sol,obj_sol,status ] = glpk(OBJ , new_S , Bvector , new_LB , new_UB , CTYPE, VARTYPE , min_max , param);	

    v_ex = v_sol(1:N);
    ex_mets = ex_mets( logical( v_sol( (1:t) + ( N + 4*t) ) ));

end

