function [exchanged v_flux status ex_rxn_list] = min_sum_exchange( model )
%[exchange v_flux status] = min_sum_exchange( model )
% Takes as input : (or model must have )
%   model
%	    .S 		- The stoichiometric matrix, rows are metabolites and columns are reactions
%	    .Ubound	- A vector the same length as the number of reactions, indicating the upper bound for each
%	    .Lbound	- A vector the same length as the number of reactions, indicating the lower bound for each
%	    .OBJ		- A vector the same length as the number of reactions,
%                indicating which fluxes should be optimized 
%       .t_rxns - A vector of the same length as the number of interactions, indicateing which organsims
%                 each shuttle reaction belongs to.
%   optoinal
%       .bm     - biomass value, 1/10 of this value will be imposed as a constraint, default is 1 (.1 bm units imposed)
%
% Returns : The fluxes through each reaction and the value of the optimized objective function. Note that a second
% Author: Niels
% Date:   Aug 16, 2010
% Updated:  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	

%Check if a model was passed
if ( isstruct( model ) )
    if ( isfield( model, 'S')  && isfield( model, 'lb') && isfield( model, 'ub') ...
      && isfield( model, 'c' ) && isfield( model, 't_rxns') )        
           

         % get needed fields
         S      = model.S;
         Lbound = model.lb;
         Ubound = model.ub;
         OBJ    = model.c;
         t_rxns = model.t_rxns;   
         
         MAX_FLUX = max(Ubound);

    else
         error( 'Unrecognized model: must contain S, lb, ub and c fields' );
    end
end

% fix minimal amount of biomass prodcution...
if isfield( model, 'bm' )
    BM = model.bm;
else
    BM = 1;
end

Lbound(logical(OBJ)) = BM/10;

%NUMBER OF METABOLITES PLUS ADDITIONAL CONSTRAINTS(ROWS OF S, CONSTRAINTS) => M
%NUMBER OF REACTIONS (COLUMNS OF S, VARIABLES)                             => N
[M,N] = size(S); 

%SET THE B VECTOR FOR THE RIGHT HAND SIDE OF EQUATION, Sv= 0
Bvector = zeros(M, 1);

	
%% SET PARAMETERS FOR THE LP OPTIMIZATRION SOLVE %%

%IF SET TO 3, PRINTS OUT DETAILS OF OPTIMIZATION PROCESS
param.msglev = 1;
	
% IMPORTANT: THIS, WHEN SET TO 2, USES INTERIOR POINT 
param.LPsolver = 1; 

%IF 1, SAVE PROBLEM IN LP FORMAT
param.save_pb = 0; 

%TURN OFF THE PRESOLVER
param.presol = 0;


%% FIND SOLUTION WHICH MINIMIZES SUM OF ABSOLUTE VALUES OF THE FLUXES GIVEN SOME growth%%

%don't include mets from current medium!

[~,current_m] = get_environment( model );
for i=1:length( current_m )
    met = find(S(:,current_m(i)));
    met_rxns = find( S(met,:));
    t_rxns(met_rxns) = 0;
end


num_rxns = length(find(t_rxns));
S_eye = eye(N);
 
S = [S zeros(M,num_rxns); ...
        S_eye(logical(t_rxns),:) -1*eye(num_rxns); ...
     -1*S_eye(logical(t_rxns),:) -1*eye(num_rxns)];
 
 
Lbound  = [Lbound; zeros(num_rxns,1)];
Ubound  = [Ubound; MAX_FLUX*ones(num_rxns,1)];
OBJ     = [zeros(N,1); ones(num_rxns,1)];
Bvector = [Bvector ; zeros(2*num_rxns,1)];	
	
original_N = N;
[~,N] = size(S);

%MINIMIZE SUM OF ABSOLUTE VALUES OF FLUXES
min_max = 1;


%C-TYPE FOR NEW CONSTRAINTS HAS UPPER BOUND
CTYPE = [  char('S'*ones(M,1)); char( 'U'*ones( num_rxns*2, 1) )];


%VARTYPES STILL ALL CONTINUOUS 
VARTYPE = char( 'C'*ones(N,1) ); 


%PERFORM OPTIMIZATION
[v_sol_minAV ,~,status ] = ...
    glpk(OBJ, S, Bvector, Lbound, Ubound, CTYPE, VARTYPE, min_max, param);

v_flux = v_sol_minAV( 1:original_N );
 
%find exchange fluxes...
pos_org1_ex = intersect(find(t_rxns == 1), find( v_flux >  1e-6) );
neg_org1_ex = intersect(find(t_rxns == 1), find( v_flux < -1e-6) );
   
pos_org2_ex = intersect(find(t_rxns == 2), find( v_flux >  1e-6) );
neg_org2_ex = intersect(find(t_rxns == 2), find( v_flux < -1e-6) );

exchanged = [];
ex_rxn_list = zeros(0,2);
ex_count = 0;
for i=1:length(pos_org1_ex)
    i_mets = find( model.S(:,pos_org1_ex(i)));
    for j =1:length(neg_org2_ex)
        j_mets = find( model.S(:,neg_org2_ex(j)));
        ex_met = intersect( i_mets, j_mets );
        if (ex_met)
            exchanged = union(exchanged, ex_met);
            ex_count = ex_count +1;
            ex_rxn_list(ex_count,1) = pos_org1_ex(i);
            ex_rxn_list(ex_count,2) = neg_org2_ex(j);
        end
        
    end
end

for i=1:length(pos_org2_ex)
    i_mets = find( model.S(:,pos_org2_ex(i)));
    for j =1:length(neg_org1_ex)
        j_mets = find( model.S(:,neg_org1_ex(j)));
        ex_met = intersect( i_mets, j_mets );
        if (ex_met)
            exchanged = union(exchanged, ex_met);
            ex_count = ex_count +1;
            ex_rxn_list(ex_count,1) = neg_org1_ex(j);
            ex_rxn_list(ex_count,2) = pos_org2_ex(i);  
        end
        
    end
end

end

